// $Id: ComObjectFactory.h,v 1.8 2002/02/09 17:02:40 cthuang Exp $
#ifndef COMOBJECTFACTORY_H
#define COMOBJECTFACTORY_H

#include "tcomApi.h"
#include "mutex.h"
#include "TclObject.h"
#include "TypeInfo.h"

// This is a factory of COM objects.

class TCOM_API ComObjectFactory: public IClassFactory
{
    // reference count of the factory
    long m_refCount;

    // interfaces to implement
    const Class::Interfaces &m_interfaces;

    // thread running the Tcl interpreter to use
    Tcl_ThreadId m_threadId;
                                            
    // Tcl interpreter used to execute Tcl commands
    Tcl_Interp *m_interp;

    // Tcl command executed to create a servant
    TclObject m_constructor;

    // Tcl command executed to destroy servant
    TclObject m_destructor;

    // handle of registered class object
    unsigned long m_classObjectHandle;

    // CLSID used to register active object
    CLSID m_clsid;

    // true if created objects should be registered in running object table
    bool m_registerActiveObject;

    // true if object factory was registered
    bool m_registeredFactory;

public:
    ComObjectFactory(
        const Class::Interfaces &interfaces,
        Tcl_ThreadId threadId,
        Tcl_Interp *interp,
        TclObject constructor,
        TclObject destructor,
        bool registerActiveObject);
    virtual ~ComObjectFactory();

    // Register factory.
    void registerFactory(REFCLSID clsid, DWORD regclsFlags);

    // IUnknown methods
    STDMETHOD(QueryInterface)(REFIID riid, void **ppvObj);
    STDMETHOD_(ULONG, AddRef)();
    STDMETHOD_(ULONG, Release)();

    // IClassFactory methods
    STDMETHOD(CreateInstance)(IUnknown *pOuter, REFIID riid, void **ppvObj);
    STDMETHOD(LockServer)(BOOL fLock);
};

// This is factory returns always the same instance.

class TCOM_API SingletonObjectFactory: public ComObjectFactory
{
    DECLARE_MUTEX(m_mutex)

    IUnknown *m_pInstance;

public:
    SingletonObjectFactory(
        const Class::Interfaces &interfaces,
        Tcl_ThreadId threadId,
        Tcl_Interp *interp,
        TclObject constructor,
        TclObject destructor,
        bool registerActiveObject);
    ~SingletonObjectFactory();

    // Override create function.
    STDMETHOD(CreateInstance)(IUnknown *pOuter, REFIID riid, void **ppvObj);
};

#endif 
