// $Id: CommandProcessor.h,v 1.10 2001/06/23 17:09:38 cthuang Exp $
#ifndef COMMANDPROCESSOR_H
#define COMMANDPROCESSOR_H

#include <queue>
#include <tcl.h>
#include "tcomApi.h"
#include "Singleton.h"

class TclObject;
struct Command;

// This class provides the ability to execute Tcl commands in a specific
// thread and interpreter.

class TCOM_API CommandProcessor
{
    // queue of commands to execute
    typedef std::queue<Command *> Queue;
    Queue m_queue;

    // used to notify there is an incoming method invocation
    Tcl_AsyncHandler m_asyncHandler;

    friend class Singleton<CommandProcessor>;
    static Singleton<CommandProcessor> ms_singleton;

    static int asyncProc(ClientData clientData, Tcl_Interp *interp, int code);
    static int eventProc(Tcl_Event *pEvent, int flags);

    // Do not allow others to create or copy instances of this class.
    CommandProcessor();
    CommandProcessor(const CommandProcessor &);     // not implemented
    void operator=(const CommandProcessor &);       // not implemented

public:
    ~CommandProcessor();

    // Get singleton instance.
    static CommandProcessor &instance();

    // Execute a Tcl command.  If pResult is not null, wait for the command
    // to complete and get the result.  Return a Tcl completion code.
    int eval(
        Tcl_ThreadId threadId,
        Tcl_Interp *interp,
        TclObject script,
        TclObject *pResult=0);
};

#endif
