// $Id: TclInterp.cpp,v 1.9 2001/11/07 18:00:23 cthuang Exp $
#include <stdexcept>
#include "TclInterp.h"

TclInterp::TclInterp ():
    m_interp(0)
{ }

TclInterp::~TclInterp ()
{
    // TODO: I would like to clean up resources here, but when the destructor
    // is called on static instances of this class, the Tcl memory subsystem
    // may have been finalized, making the argument point to an invalid memory
    // address.
//    Tcl_DeleteInterp(m_interp);
}

void
TclInterp::doInitialize (const std::string &dllPath)
{
    // Load Tcl library.
    _bstr_t path(dllPath.c_str());
    m_hmodTcl = CoLoadLibrary(path, TRUE);
    if (m_hmodTcl == NULL) {
        throw std::runtime_error("CoLoadLibrary");
    }

    // Get address of Tcl_CreateInterp function.
    typedef Tcl_Interp *(*CreateInterpFunc)();
    CreateInterpFunc createInterp = reinterpret_cast<CreateInterpFunc>(
        GetProcAddress(m_hmodTcl, "Tcl_CreateInterp"));
    if (createInterp == NULL) {
        throw std::runtime_error("GetProcAddress");
    }

    // Create Tcl interpreter.
    m_interp = createInterp();
    if (Tcl_InitStubs(m_interp, "8.1", 0) == NULL) {
        throw std::runtime_error("Tcl_InitStubs");
    }

    Tcl_FindExecutable(dllPath.c_str());

    // Find and source Tcl initialization script.
    if (Tcl_Init(m_interp) != TCL_OK) {
        throw std::runtime_error(Tcl_GetStringResult(m_interp));
    }
}

void
TclInterp::initialize (const std::string &dllPath)
{
    if (m_interp == 0) {
        doInitialize(dllPath);
    }
}
