// $Id: TypeLib.h,v 1.19 2001/11/28 17:11:27 cthuang Exp $
#ifndef TYPELIB_H
#define TYPELIB_H

#include <vector>
#include <map>
#include <string>
#include "TypeInfo.h"

// This describes an enumeration with a map where
// the key is an enumerator name and
// the data is the enumerator value.

class Enum: public std::map<std::string, std::string>
{
    std::string m_name;

public:
    Enum(TypeInfo &typeInfo, TypeAttr &attr);

    // Get name.
    const std::string &name () const
    { return m_name; }
};

// This class represents a type library.

class TypeLib
{
public:
    typedef std::vector<const Interface *> Interfaces;
    typedef std::vector<Class> Classes;
    typedef std::vector<Enum> Enums;

private:
    ITypeLib *m_pITypeLib;
    Interfaces m_interfaces;
    Classes m_classes;
    Enums m_enums;

    TypeLib (ITypeLib *pITypeLib):
        m_pITypeLib(pITypeLib)
    { readTypeLib(); }

    // Do not allow others to copy instances of this class.
    TypeLib(const TypeLib &);           // not implemented
    void operator=(const TypeLib &);    // not implemented

    // Get information from type library.
    void readTypeLib();

public:
    ~TypeLib ()
    { m_pITypeLib->Release(); }

    // Load a type library from the specified file.
    static TypeLib *load(const char *name, bool registerTypeLib=false);

    // Load a type library specified by a LIBID.
    // Return 0 if the required registry entries were not found.
    static TypeLib *loadByLibid(
        const std::string &libid, const std::string &version);

    // Load a type library specified by a CLSID.
    // Return 0 if the required registry entries were not found.
    static TypeLib *loadByClsid(REFCLSID clsid);

    // Load a type library specified by an IID.
    // Return 0 if the required registry entries were not found.
    static TypeLib *loadByIid(REFIID iid);

    // Get string representation of type library ID.
    std::string libidString() const;

    // Get type library version.
    std::string version() const;

    // Get type library name.
    std::string name() const;

    // Get type library documentation string.
    std::string documentation() const;

    // Get interfaces.
    const Interfaces &interfaces () const
    { return m_interfaces; }

    // Get the named interface.
    const Interface *findInterface(const char *name) const;

    // Get classes.
    const Classes &classes () const
    { return m_classes; }

    // Get the named class.
    const Class *findClass(const char *name) const;

    // Find class by CLSID.
    const Class *findClass(REFCLSID clsid) const;

    // Get enumerations.
    const Enums &enums () const
    { return m_enums; }

    // Get the named enumeration.
    const Enum *findEnum(const char *name) const;
};

#endif 
