// $Id: main.cpp,v 1.61 2002/02/18 23:10:14 cthuang Exp $
#pragma warning(disable: 4786)
#include "ComInitializer.h"
#include "bindCmd.h"
#include "configureCmd.h"
#include "foreachCmd.h"
#include "importCmd.h"
#include "infoCmd.h"
#include "naCmd.h"
#include "nullCmd.h"
#include "objectCmd.h"
#include "refCmd.h"
#include "shortPathNameCmd.h"
#include "typelibCmd.h"

#define NAMESPACE "::tcom::"
#define PACKAGE_NAME "tcom"
#define PACKAGE_VERSION "3.7"

#ifdef __BORLANDC__
#undef DLLEXPORT
#define DLLEXPORT __declspec(dllexport)
#endif

/*
 *	This procedure performs application-specific initialization.
 *	Most applications, especially those that incorporate additional
 *	packages, will have their own version of this procedure.
 *
 * Results:
 *	Returns a standard Tcl completion code, and leaves an error
 *	message in interp->result if an error occurs.
 *
 * Side effects:
 *	Depends on the startup script.
 */
extern "C" DLLEXPORT int
Tcom_Init (Tcl_Interp *interp)
{
#if TCL_MINOR_VERSION >= 1
    // Stubs were introduced in Tcl 8.1.
    if (Tcl_InitStubs(interp, "8.1", 0) == NULL) {
        return TCL_ERROR;
    }
#endif

    ComInitializer::concurrencyModel(COINIT_APARTMENTTHREADED);

    // Initialize handle support.
    CmdNameType::instance();

    // Register new Tcl types.
    Tcl_RegisterObjType(&unknownPointerType);
    Tcl_RegisterObjType(&naType);
    Tcl_RegisterObjType(&nullType);

    // Create additional commands.
    Tcl_CreateObjCommand(interp, NAMESPACE "bind", bindCmd, 0, 0);
    Tcl_CreateObjCommand(interp, NAMESPACE "class", classCmd, 0, 0);
    Tcl_CreateObjCommand(interp, NAMESPACE "configure", configureCmd, 0, 0);
    Tcl_CreateObjCommand(interp, NAMESPACE "foreach", foreachCmd, 0, 0);
    Tcl_CreateObjCommand(interp, NAMESPACE "import", importCmd, 0, 0);
    Tcl_CreateObjCommand(interp, NAMESPACE "info", infoCmd, 0, 0);
    Tcl_CreateObjCommand(interp, NAMESPACE "interface", interfaceCmd, 0, 0);
    Tcl_CreateObjCommand(interp, NAMESPACE "method", methodCmd, 0, 0);
    Tcl_CreateObjCommand(interp, NAMESPACE "na", naCmd, 0, 0);
    Tcl_CreateObjCommand(interp, NAMESPACE "null", nullCmd, 0, 0);
    Tcl_CreateObjCommand(interp, NAMESPACE "property", propertyCmd, 0, 0);
    Tcl_CreateObjCommand(interp, NAMESPACE "ref", refCmd, 0, 0);
    Tcl_CreateObjCommand(
        interp, NAMESPACE "shortPathName", shortPathNameCmd, 0, 0);
    Tcl_CreateObjCommand(interp, NAMESPACE "typelib", typelibCmd, 0, 0);
    Tcl_CreateObjCommand(interp, NAMESPACE "unbind", unbindCmd, 0, 0);
#if TCL_MINOR_VERSION >= 1
    // Server objects require the thread support introduced in Tcl 8.1.
    Tcl_CreateObjCommand(interp, NAMESPACE "object", objectCmd, 0, 0);
#endif

    return Tcl_PkgProvide(interp, PACKAGE_NAME, PACKAGE_VERSION);
}

/*
 *	This procedure initializes commands for a safe interpreter.
 *	You would leave out of this procedure any commands you deemed unsafe.
 *
 * Results:
 *	A standard Tcl result.
 *
 * Side effects:
 *	None.
 */
extern "C" DLLEXPORT int
Tcom_SafeInit (
    Tcl_Interp *interp)
{
#if TCL_MINOR_VERSION >= 1
    // Stubs were introduced in Tcl 8.1.
    if (Tcl_InitStubs(interp, "8.1", 0) == NULL) {
        return TCL_ERROR;
    }
#endif

    return Tcl_PkgProvide(interp, PACKAGE_NAME, PACKAGE_VERSION);
}
