// $Id: mutex.h,v 1.5 2001/06/23 17:09:38 cthuang Exp $
#ifndef MUTEX_H
#define MUTEX_H

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#ifdef TCL_THREADS

// This class is used for mutual-exclusion synchronization.

class Mutex
{
    CRITICAL_SECTION m_mutex;

public:
    Mutex ()
    { InitializeCriticalSection(&m_mutex); }

    ~Mutex ()
    { DeleteCriticalSection(&m_mutex); }

    void enter ()
    { EnterCriticalSection(&m_mutex); }

    void leave ()
    { LeaveCriticalSection(&m_mutex); }
};

// This class locks a mutex when constructed and unlocks it when destroyed.

class SingleLock
{
    Mutex &m_mutex;

public:
    SingleLock (Mutex &mutex):
        m_mutex(mutex)
    { m_mutex.enter(); }

    ~SingleLock ()
    { m_mutex.leave(); }
};

#define DECLARE_MUTEX(name) Mutex name;
#define LOCK_MUTEX(mutex) \
    SingleLock criticalSectionLock(const_cast<Mutex &>(mutex));

#else

#define DECLARE_MUTEX(name)
#define LOCK_MUTEX(mutex)

#endif

#endif
